{****************************************************************************}
{ This unit demonstrates a method by which one can automate a Turbo Vision   }
{ application using the event queue defined in EVENTQ.PAS. It declares a     }
{ a descendant of TEventQApp which, in addition to functioning as a near-    }
{ equivalent of TApplication, also has the ability to read keystrokes out of }
{ a script file. In order to run the demo, you need to make a copy of        }
{ TVDEMO.PAS and make the following changes:                                 }
{                                                                            }
{    1) Add 'ScrDemo' to the 'uses' statement.                               }
{    2) Change all occurrences of 'TApplication' to 'TScriptApp' (5 total).  }
{                                                                            }
{ After making the changes, compile the modified copy of TVDEMO to disk and  }
{ run it from the command line. Be sure that SCRIPT.SCR and SCRDEMO.PAS are  }
{ both present in the current directory before you run the demo.             }
{                                                                            }
{ This unit is intended to be a demo only. You will probably need to make    }
{ changes, unless the only thing you want to do is animate a Turbo Vision    }
{ program (as a self-running demo, for example). See SCRIPT.SCR for details  }
{ on the syntax of the script language.                                      }
{****************************************************************************}

{$X+}
unit ScrDemo;

interface

uses Dos,Drivers,Objects,MsgBox,EventQ;

{****************************************************************************}
{ The name of the script file is hard-coded as shown below. For a real       }
{ application, you will probably want to make it a parameter passed to Init. }
{****************************************************************************}

const
  ScriptFileName: PathStr = ('SCRIPT.SCR');

type
  PScript = ^TScript;
  TScript = object (TObject)
    F: Text;
    FileIsOpen: Boolean;
    constructor Init (FileName: PathStr);
    procedure Run (P: PEventQApp);
    destructor Done; virtual;
    end;

  PScriptApp = ^TScriptApp;
  TScriptApp = object (TEventQApp)
    Scr: TScript;
    constructor Init;
    procedure Idle; virtual;
    destructor Done; virtual;
    end;

implementation

const
  WaitTime: LongInt = (0);
  DefaultWaitTime: LongInt = (0);
  WaitStart: LongInt = (0);
  Waiting: Boolean = (False);

{****************************************************************************}
{ ScanCodes is an array which correlates the ASCII characters with the       }
{ keyboard scan codes that would have been generated, had the characters     }
{ really come from the keyboard. In some cases, more than one scan code will }
{ result in the same character; in such cases, one of the possible scan      }
{ codes was arbitrarily chosed for this array.                               }
{****************************************************************************}

  ScanCodes: array[#0..#255] of Byte = (
    $79,$1E,$30,$2E,$20,$12,$21,$22,$23,$17,$24,$25,$26,$32,$31,$18,
    $19,$10,$13,$1F,$14,$16,$2F,$11,$2D,$15,$2C,$1A,$2B,$1B,$07,$0C,
    $39,$02,$28,$04,$05,$06,$08,$28,$0A,$0B,$09,$0D,$33,$0C,$34,$35,
    $0B,$02,$03,$04,$05,$06,$07,$08,$09,$0A,$27,$27,$33,$0D,$34,$35,
    $03,$1E,$30,$2E,$20,$12,$21,$22,$23,$17,$24,$25,$26,$32,$31,$18,
    $19,$10,$13,$1F,$14,$16,$2F,$11,$2D,$15,$2C,$1A,$2B,$1B,$07,$0C,
    $29,$1E,$30,$2E,$20,$12,$21,$22,$23,$17,$24,$25,$26,$32,$31,$18,
    $19,$10,$13,$1F,$14,$16,$2F,$11,$2D,$15,$2C,$1A,$2B,$1B,$29,$00,
    $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,
    $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,
    $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,
    $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,
    $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,
    $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,
    $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,
    $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00);

function StripSpaces (S: String): String;

begin
while S[1] = ' ' do Delete (S,1,1);
while S[Length (S)] = ' ' do Delete (S,Length (S),1);
StripSpaces := S;
end;

function Upper (S: String): String;

var
  I: Integer;

begin
for I := 1 to Length (S) do S[I] := UpCase (S[I]);
Upper := S;
end; {Upper}

{****************************************************************************}
{ The huge TranslateKey procedure converts key commands in the script file   }
{ into the equivalent Turbo Vision events.                                   }
{****************************************************************************}

procedure TranslateKey (S: String; var Event: TEvent);

var
  Value,ErrCode: Integer;

begin
Event.What := evKeyDown;
if Length (S) = 1 then
  begin
  Event.CharCode := S[1];
  Event.ScanCode := ScanCodes[S[1]];
  end
else if S[1] = '^' then
  begin
  if S[1] = '@' then
    begin
    Event.CharCode := #0;
    Event.ScanCode := ScanCodes[#0];
    end
  else if S[1] in ['A'..'_'] then
    begin
    Event.CharCode := Char (Byte (S[1]) - Byte ('A'));
    Event.ScanCode := ScanCodes[Char (Byte (S[1]) - Byte ('A'))];
    end
  else if S[1] in ['a'..'z'] then
    begin
    Event.CharCode := Char (Byte (S[1]) - Byte ('a'));
    Event.ScanCode := ScanCodes[Char (Byte (S[1]) - Byte ('a'))];
    end
  else Event.What := evNothing;
  end
else if S[1] = '#' then
  begin
  Delete (S,1,1);
  Val (S,Value,ErrCode);
  if (ErrCode = 0) and (Value >= 0) and (Value <= 255) then
    begin
    Event.CharCode := Char (Value);
    Event.ScanCode := ScanCodes[Event.CharCode];
    end
  else Event.What := evNothing;
  end
else if (S[1] in ['K','k']) and (S[2] in ['B','b']) then
  begin
  Delete (S,1,2);
  S := Upper (S);
  if S = 'ALTA' then Event.KeyCode := $1E00
  else if S = 'ALTB' then Event.KeyCode := $3000
  else if S = 'ALTC' then Event.KeyCode := $2E00
  else if S = 'ALTD' then Event.KeyCode := $2000
  else if S = 'ALTE' then Event.KeyCode := $1200
  else if S = 'ALTF' then Event.KeyCode := $2100
  else if S = 'ALTG' then Event.KeyCode := $2200
  else if S = 'ALTH' then Event.KeyCode := $2300
  else if S = 'ALTI' then Event.KeyCode := $1700
  else if S = 'ALTJ' then Event.KeyCode := $2400
  else if S = 'ALTK' then Event.KeyCode := $2500
  else if S = 'ALTL' then Event.KeyCode := $2600
  else if S = 'ALTM' then Event.KeyCode := $3200
  else if S = 'ALTN' then Event.KeyCode := $3100
  else if S = 'ALTO' then Event.KeyCode := $1800
  else if S = 'ALTP' then Event.KeyCode := $1900
  else if S = 'ALTQ' then Event.KeyCode := $1000
  else if S = 'ALTR' then Event.KeyCode := $1300
  else if S = 'ALTS' then Event.KeyCode := $1F00
  else if S = 'ALTT' then Event.KeyCode := $1400
  else if S = 'ALTU' then Event.KeyCode := $1600
  else if S = 'ALTV' then Event.KeyCode := $2F00
  else if S = 'ALTW' then Event.KeyCode := $1100
  else if S = 'ALTX' then Event.KeyCode := $2D00
  else if S = 'ALTY' then Event.KeyCode := $1500
  else if S = 'ALTZ' then Event.KeyCode := $2C00
  else if S = 'ALT1' then Event.KeyCode := $7800
  else if S = 'ALT2' then Event.KeyCode := $7900
  else if S = 'ALT3' then Event.KeyCode := $7A00
  else if S = 'ALT4' then Event.KeyCode := $7B00
  else if S = 'ALT5' then Event.KeyCode := $7C00
  else if S = 'ALT6' then Event.KeyCode := $7D00
  else if S = 'ALT7' then Event.KeyCode := $7E00
  else if S = 'ALT8' then Event.KeyCode := $7F00
  else if S = 'ALT9' then Event.KeyCode := $8000
  else if S = 'ALT0' then Event.KeyCode := $8100
  else if S = 'F1' then Event.KeyCode := $3B00
  else if S = 'F2' then Event.KeyCode := $3C00
  else if S = 'F3' then Event.KeyCode := $3D00
  else if S = 'F4' then Event.KeyCode := $3E00
  else if S = 'F5' then Event.KeyCode := $3F00
  else if S = 'F6' then Event.KeyCode := $4000
  else if S = 'F7' then Event.KeyCode := $4100
  else if S = 'F8' then Event.KeyCode := $4200
  else if S = 'F9' then Event.KeyCode := $4300
  else if S = 'F10' then Event.KeyCode := $4400
  else if S = 'SHIFTF1' then Event.KeyCode := $5400
  else if S = 'SHIFTF2' then Event.KeyCode := $5500
  else if S = 'SHIFTF3' then Event.KeyCode := $5600
  else if S = 'SHIFTF4' then Event.KeyCode := $5700
  else if S = 'SHIFTF5' then Event.KeyCode := $5800
  else if S = 'SHIFTF6' then Event.KeyCode := $5900
  else if S = 'SHIFTF7' then Event.KeyCode := $5A00
  else if S = 'SHIFTF8' then Event.KeyCode := $5B00
  else if S = 'SHIFTF9' then Event.KeyCode := $5C00
  else if S = 'SHIFTF10' then Event.KeyCode := $5D00
  else if S = 'CTRLF1' then Event.KeyCode := $5E00
  else if S = 'CTRLF2' then Event.KeyCode := $5F00
  else if S = 'CTRLF3' then Event.KeyCode := $6000
  else if S = 'CTRLF4' then Event.KeyCode := $6100
  else if S = 'CTRLF5' then Event.KeyCode := $6200
  else if S = 'CTRLF6' then Event.KeyCode := $6300
  else if S = 'CTRLF7' then Event.KeyCode := $6400
  else if S = 'CTRLF8' then Event.KeyCode := $6500
  else if S = 'CTRLF9' then Event.KeyCode := $6600
  else if S = 'CTRLF10' then Event.KeyCode := $6700
  else if S = 'ALTF1' then Event.KeyCode := $6800
  else if S = 'ALTF2' then Event.KeyCode := $6900
  else if S = 'ALTF3' then Event.KeyCode := $6A00
  else if S = 'ALTF4' then Event.KeyCode := $6B00
  else if S = 'ALTF5' then Event.KeyCode := $6C00
  else if S = 'ALTF6' then Event.KeyCode := $6D00
  else if S = 'ALTF7' then Event.KeyCode := $6E00
  else if S = 'ALTF8' then Event.KeyCode := $6F00
  else if S = 'ALTF9' then Event.KeyCode := $7000
  else if S = 'ALTF10' then Event.KeyCode := $7100
  else if S = 'ALTEQUAL' then Event.KeyCode := $8300
  else if S = 'ALTMINUS' then Event.KeyCode := $8200
  else if S = 'ALTSPACE' then Event.KeyCode := $0200
  else if S = 'BACK' then Event.KeyCode := $0E08
  else if S = 'CTRLBACK' then Event.KeyCode := $0E7F
  else if S = 'CTRLDEL' then Event.KeyCode := $0600
  else if S = 'CTRLEND' then Event.KeyCode := $7500
  else if S = 'CTRLENTER' then Event.KeyCode := $1C0A
  else if S = 'CTRLHOME' then Event.KeyCode := $7700
  else if S = 'CTRLINS' then Event.KeyCode := $0400
  else if S = 'CTRLLEFT' then Event.KeyCode := $7300
  else if S = 'CTRLPGDN' then Event.KeyCode := $7600
  else if S = 'CTRLPGUP' then Event.KeyCode := $8400
  else if S = 'CTRLPRTSC' then Event.KeyCode := $7200
  else if S = 'CTRLRIGHT' then Event.KeyCode := $7400
  else if S = 'DEL' then Event.KeyCode := $5300
  else if S = 'DOWN' then Event.KeyCode := $5000
  else if S = 'END' then Event.KeyCode := $4F00
  else if S = 'ENTER' then Event.KeyCode := $1C0D
  else if S = 'ESC' then Event.KeyCode := $011B
  else if S = 'GRAYMINUS' then Event.KeyCode := $4A2D
  else if S = 'HOME' then Event.KeyCode := $4700
  else if S = 'INS' then Event.KeyCode := $5200
  else if S = 'LEFT' then Event.KeyCode := $4B00
  else if S = 'PGDN' then Event.KeyCode := $5100
  else if S = 'PGUP' then Event.KeyCode := $4900
  else if S = 'GRAYPLUS' then Event.KeyCode := $4E2B
  else if S = 'RIGHT' then Event.KeyCode := $4D00
  else if S = 'SHIFTDEL' then Event.KeyCode := $0700
  else if S = 'SHIFTINS' then Event.KeyCode := $0500
  else if S = 'SHIFTTAB' then Event.KeyCode := $0F00
  else if S = 'TAB' then Event.KeyCode := $0F09
  else if S = 'UP' then Event.KeyCode := $4800
  else Event.What := evNothing;
  end;
end;  {TranslateKey}

procedure GetNextEvent (var F: Text; P: PEventQApp);

var
  Event: TEvent;
  S: String;
  Time: LongInt;
  ErrCode: Integer;

begin
ReadLn (F,S);
case S[1] of
  'K','k': begin
    Delete (S,1,1);
    S := StripSpaces (S);
    TranslateKey (S,Event);
    P^.PushKey (Event);
    end;
  'T','t': begin
    Delete (S,1,1);
    S := StripSpaces (S);
    Val (S,Time,ErrCode);
    if ErrCode = 0 then DefaultWaitTime := Time;
    WaitTime := DefaultWaitTime;
    end;
  'W','w': begin
    Delete (S,1,1);
    S := StripSpaces (S);
    Val (S,Time,ErrCode);
    if ErrCode = 0 then WaitTime := Time;
    end;
  end;
end;  {GetNextEvent}

function GetDosTime: LongInt;

var
  Hour,Minute,Second,Sec100: Word;

begin
GetTime (Hour,Minute,Second,Sec100);
GetDosTime := 10 * (Sec100 + 100 * (Second + 60 * (Minute
  + LongInt (60) * Hour)));
end;  {GetDosTime}

constructor TScript.Init (FileName: PathStr);

begin
FileName := FExpand (FileName);
Assign (F,FileName);
FileIsOpen := True;
{$I-}
Reset (F);
{$I+}
if IOResult <> 0 then
  begin
  FileIsOpen := False;
  MessageBox ('Can''t find script file ' + FileName + '.',nil,
    mfError + mfOKButton);
  end;
end;  {TScript.Init}

procedure TScript.Run (P: PEventQApp);

begin
if not Waiting then
  begin
  WaitStart := GetDosTime;
  Waiting := True;
  end
else begin
  if GetDosTime > WaitStart + WaitTime then
    begin
    WaitTime := DefaultWaitTime;
    if FileIsOpen and (not Eof (F)) then GetNextEvent (F,P);
    Waiting := False;
    end;
  end;
end;  {TScript.Run}

destructor TScript.Done;

begin
if FileIsOpen then Close (F);
FileIsOpen := False;
TObject.Done;
end;  {TScript.Done}

constructor TScriptApp.Init;

begin
TEventQApp.Init;
Scr.Init (ScriptFileName);
end;  {TScriptApp.Init}

procedure TScriptApp.Idle;

begin
TEventQApp.Idle;
Scr.Run (@Self);
end;  {TScriptApp.Idle}

destructor TScriptApp.Done;

begin
Scr.Done;
TEventQApp.Done;
end;  {TScriptApp.Done}

end.  {ScrDemo}
